@@ -25,8 +25,8 @@ int server_socket;
 static char getopt_env[] = "POSIXLY_CORRECT=YES";
 static char *old_getopt_env;
 
-static char version[] = "Task Spooler v0.7.0 - a task queue system for the unix user.\n"
-"Copyright (C) 2007-2009  Lluis Batlle i Rossell";
+static char version[] = "Task Spooler v1.0.2 - a task queue system for the unix user.\n"
+"Copyright (C) 2007-2020  Lluis Batlle i Rossell";
 
 
 static void default_command_line()
@@ -43,6 +43,8 @@ static void default_command_line()
     command_line.depend_on = -1; /* -1 means depend on previous */
     command_line.max_slots = 1;
     command_line.wait_enqueuing = 1;
+    command_line.stderr_apart = 0;
+    command_line.num_slots = 1;
 }
 
 void get_command(int index, int argc, char **argv)
@@ -82,7 +84,7 @@ void parse_opts(int argc, char **argv)
 
     /* Parse options */
     while(1) {
-        c = getopt(argc, argv, ":VhKgClnfmBr:t:c:o:p:w:u:s:U:i:L:dS:D:");
+        c = getopt(argc, argv, ":VhKgClnfmBEr:t:c:o:p:w:k:u:s:U:i:N:L:dS:D:");
 
         if (c == -1)
             break;
@@ -93,6 +95,10 @@ void parse_opts(int argc, char **argv)
                 command_line.request = c_KILL_SERVER;
                 command_line.should_go_background = 0;
                 break;
+            case 'k':
+                command_line.request = c_KILL_JOB;
+                command_line.jobid = atoi(optarg);
+                break;
             case 'l':
                 command_line.request = c_LIST;
                 break;
@@ -144,6 +150,11 @@ void parse_opts(int argc, char **argv)
                 command_line.request = c_INFO;
                 command_line.jobid = atoi(optarg);
                 break;
+            case 'N':
+                command_line.num_slots = atoi(optarg);
+                if (command_line.num_slots < 0)
+                    command_line.num_slots = 0;
+                break;
             case 'r':
                 command_line.request = c_REMOVEJOB;
                 command_line.jobid = atoi(optarg);
@@ -193,6 +204,9 @@ void parse_opts(int argc, char **argv)
                 /* I picked 'B' quite at random among the letters left */
                 command_line.wait_enqueuing = 0;
                 break;
+            case 'E':
+                command_line.stderr_apart = 1;
+                break;
             case ':':
                 switch(optopt)
                 {
@@ -236,6 +250,10 @@ void parse_opts(int argc, char **argv)
                         command_line.jobid = -1; /* This means the 'last'
                                                     added job */
                         break;
+                    case 'k':
+                        command_line.request = c_KILL_JOB;
+                        command_line.jobid = -1; /* This means the 'last' job */
+                        break;
                     case 'S':
                         command_line.request = c_GET_MAX_SLOTS;
                         break;
@@ -317,7 +335,7 @@ static void go_background()
 
 static void print_help(const char *cmd)
 {
-    printf("usage: %s [action] [-ngfmd] [-L <lab>] [-D <id>] [cmd...]\n", cmd);
+    printf("usage: %s [action] [-ngfmdE] [-L <lab>] [-D <id>] [cmd...]\n", cmd);
     printf("Env vars:\n");
     printf("  TS_SOCKET  the path to the unix socket used by the ts command.\n");
     printf("  TS_MAILTO  where to mail the result (on -m). Local user by default.\n");
@@ -327,6 +345,7 @@ static void print_help(const char *cmd)
     printf("  TS_ENV  command called on enqueue. Its output determines the job information.\n");
     printf("  TS_SAVELIST  filename which will store the list, if the server dies.\n");
     printf("  TS_SLOTS   amount of jobs which can run at once, read on server start.\n");
+    printf("  TMPDIR     directory where to place the output files and the default socket.\n");
     printf("Actions:\n");
     printf("  -K       kill the task spooler server\n");
     printf("  -C       clear the list of finished jobs\n");
@@ -340,6 +359,7 @@ static void print_help(const char *cmd)
     printf("  -s [id]  show the job state. Of the last added, if not specified.\n");
     printf("  -r [id]  remove a job. The last added, if not specified.\n");
     printf("  -w [id]  wait for a job. The last added, if not specified.\n");
+    printf("  -k [id]  send SIGTERM to the job process group. The last run, if not specified.\n");
     printf("  -u [id]  put that job first. The last added, if not specified.\n");
     printf("  -U <id-id>  swap two jobs in the queue.\n");
     printf("  -B       in case of full queue on the server, quit (2) instead of waiting.\n");
@@ -347,12 +367,14 @@ static void print_help(const char *cmd)
     printf("  -V       show the program version\n");
     printf("Options adding jobs:\n");
     printf("  -n       don't store the output of the command.\n");
+    printf("  -E       Keep stderr apart, in a name like the output file, but adding '.e'.\n");
     printf("  -g       gzip the stored output (if not -n).\n");
     printf("  -f       don't fork into background.\n");
     printf("  -m       send the output by e-mail (uses sendmail).\n");
     printf("  -d       the job will be run only if the job before ends well\n");
-    printf("  -D <id>  the job will be run only if the job of given id ends well \n");
+    printf("  -D <id>  the job will be run only if the job of given id ends well.\n");
     printf("  -L <lab> name this task with a label, to be distinguished on listing.\n");
+    printf("  -N <num> number of slots required by the job (1 default).\n");
 }
 
 static void print_version()
@@ -466,6 +488,11 @@ int main(int argc, char **argv)
             error("The command %i needs the server", command_line.request);
         c_show_pid();
         break;
+    case c_KILL_JOB:
+        if (!command_line.need_server)
+            error("The command %i needs the server", command_line.request);
+        c_kill_job();
+        break;
     case c_INFO:
         if (!command_line.need_server)
             error("The command %i needs the server", command_line.request);
